#include "StdAfx.h"
#include ".\idleplugin.h"
#include "CommonService.h"

#define TIMER_INTERVAL	(1 * 1000)
#define HOOKDLL			"hook.dll"
#define HD_HOOKHITCOUNT	"hookHitCount"
#define HD_ISFULLSCREEN	"isFullScreen"
#define HD_SETHOOK		"SetHook"
#define HD_UNSETHOOK	"UnsetHook"

enum{
	IDLE_BEGIN,
	IDLE_END,
};

CIdlePlugin CIdlePlugin::m_singleton;

//*******************************************************************
// RXgN^EfXgN^
//*******************************************************************
CIdlePlugin::CIdlePlugin(void)
: m_bEnable(false)
, m_timerCount(0)
, m_hTimer(NULL)
{
	m_pApiVer		= NSM_API_VERSION;
	m_pModuleName	= NMM_ADDIN "/" PLUGIN_NAME;
	m_pTitle		= "Idle";
	m_pDescription	= "uAChv@\ǉ܂";
	m_pAuthor		= "fors";
	m_pCopyright	= "http://cc.ii2.cc/~fors/sysworks/prog/regne/";
	m_pPluginVer	= "0.3.1";
}

CIdlePlugin::~CIdlePlugin(void)
{
}

//*******************************************************************
// I[o[Ch
//*******************************************************************
int CIdlePlugin::OnInitialize()
{
	int ret = LoadHookDll();

	if( IsDllLoadSuccess() ){
		NSM_SERVICE.CreateService(IDLE_SHOW_OPTION_DIALOG, ServiceShowOptionDialog);
		NSM_SERVICE.CreateService(IDLE_GET_IDLE_TIME, ServiceGetIdleTime);
		NSM_SERVICE.CreateService(IDLE_IS_IDLING, ServiceIsIdling);
		NSM_SERVICE.CreateService(IDLE_IS_FULLSCREEN, ServiceIsFullScreen);

		if( IDLE_SETTINGS.IsEnableRecoverName() ){
			NSM_SERVICE.HookEvent(NME_SYSTEM_CONNECTION_INFOCHANGE, HookConInfoChange);
		}
	}

	return ret;
}

int CIdlePlugin::OnSystemModulesLoaded()
{
	if( IsDllLoadSuccess() ){
		if( IDLE_SETTINGS.IsEnable() )
			Enable();

		return NMP_ALLRIGHT;
	}else{
		NSM_SERVICE.MsgBox(L"Idle", L"hook.dll̃[hɎs܂B\nIdlevOC̑SĂ̋@\łB");
		return NMP_INTERNAL_ERROR;
	}
}

void CIdlePlugin::OnTerminate()
{
	if( IsDllLoadSuccess() ){
		Disable();
		FreeHookDll();
	}
}

//*******************************************************************
// T[rX
//*******************************************************************

int CIdlePlugin::DoShowOptionDialog(HWND hWnd)
{
	Disable();
	IDLE_SETTINGS.ShowOptionDlg(GetModuleInstance(), hWnd);
	if( IDLE_SETTINGS.IsEnable() ) Enable();
	return 0;
}

int CIdlePlugin::DoGetIdleTime(void)
{
	if( ! m_bEnable ) return -1;

	return m_timerCount;
}

int CIdlePlugin::DoIsIdling(void)
{
	if( ! m_bEnable ) return -1;

	return m_cons.size() != 0;
}

int CIdlePlugin::DoIsFullScreen(void)
{
	return IsFullScreen();
}

//*******************************************************************
// Cxgnh
//*******************************************************************

int CIdlePlugin::OnConInfoChange(HNsmConnection hCon, PNsmConnectionInfo pnci)
{
	switch( pnci->nInfoKey ){
	case NMCI_STATUS:
		if( *(int*)(pnci->lpInfo->lpBuffer) == NMCS_CONNECTED ){
			RestoreNameChange(hCon);
		}
		break;
	}

	return 0;
}

//*******************************************************************
// Ǘ
//*******************************************************************

int CIdlePlugin::LoadHookDll()
{
	m_hHookDll = IDLE_SETTINGS.LoadLibrary(HOOKDLL);
	if( m_hHookDll == NULL ) return NMP_INTERNAL_ERROR;

	m_hookHitCount = (int*)::GetProcAddress(m_hHookDll, HD_HOOKHITCOUNT);
	if( m_hookHitCount == NULL ){
		FreeHookDll();
		return NMP_INTERNAL_ERROR;
	}
/*
	m_isFullScreen = (BOOL*)::GetProcAddress(m_hHookDll, HD_ISFULLSCREEN);
	if( m_isFullScreen == NULL ){
		FreeHookDll();
		return NMP_INTERNAL_ERROR;
	}
*/
	SetHook = (HookFunc)::GetProcAddress(m_hHookDll, HD_SETHOOK);
	if( SetHook == NULL ){
		FreeHookDll();
		return NMP_INTERNAL_ERROR;
	}

	UnsetHook = (HookFunc)::GetProcAddress(m_hHookDll, HD_UNSETHOOK);
	if( UnsetHook == NULL ){
		FreeHookDll();
		return NMP_INTERNAL_ERROR;
	}

	return NMP_ALLRIGHT;
}

VOID CIdlePlugin::FreeHookDll()
{
	::FreeLibrary(m_hHookDll);
	m_hHookDll = NULL;
}

// vOCJn
VOID CIdlePlugin::Enable()
{
	if( !m_bEnable ){
		m_bEnable = true;
		m_bFullScr = false;
		SetTimer();
		SetHook();
	}
}

// vOC~
VOID CIdlePlugin::Disable()
{
	if( m_bEnable ){
		KillTimer();
		UnsetHook();
		if( m_cons.size() > 0 )
			StateEnd();
		m_bEnable = false;
	}
}

// ^C}Zbg
VOID CIdlePlugin::SetTimer()
{
	m_hTimer = ::SetTimer(NULL, NULL, TIMER_INTERVAL, TimerProc);
}

// ^C}~
VOID CIdlePlugin::KillTimer()
{
	::KillTimer(NULL, m_hTimer);
	m_hTimer = NULL;
}

//*******************************************************************
// ACh
//*******************************************************************

// ^C}vV[W
VOID CIdlePlugin::OnTimer()
{
	if( IDLE_SETTINGS.IsEnableFullScr() ){
		bool bFullScr = IsFullScreen();

		if( bFullScr != m_bFullScr ){
			if( bFullScr ){
_RPT0(_CRT_WARN, "fs TRUE\n");
				EnterFullScreen();
			}else{
_RPT0(_CRT_WARN, "fs false\n");
				StateEnd();
			}
			m_bFullScr = bFullScr;
		}
	}

	if( ! m_bFullScr ){
		IdleCheck();
	}
}

// ACh`FbN
void CIdlePlugin::IdleCheck()
{
	const ConditionInfo *pci;

	IDLE_SETTINGS.LockCond();
	if( IsUserActive() )
	// [UmF
	{
		if( m_cons.size() > 0 ){
			StateEnd();
		}
		ResetUserActive();
		m_timerCount = 0;
	}
	else
	// [UACh
	{
		m_timerCount++;
		pci = IDLE_SETTINGS.GetNextCondition(m_timerCount);
		if( pci != NULL ){
			IdleBegin(pci);
		}
	}
	IDLE_SETTINGS.UnlockCond();
}

bool CIdlePlugin::IsFullScreen() const
{
	// fXNgbv`̎擾
	RECT r;
	{
		HWND hDesktop = ::GetDesktopWindow();
		if( hDesktop == NULL )
			return false;

		BOOL bSuccess;
		bSuccess = ::GetWindowRect(hDesktop, &r);
		if( ! bSuccess )
			return false;
	}


	// tHAOEhEChE̎擾
	HWND hWnd = ::GetForegroundWindow();

	// gbvxŔwʃEChEi=fXNgbvjłΖ
	if( ::GetWindow(hWnd, GW_HWNDLAST) == hWnd )
		return false;

	// tHAOEhEChEт̃I[i[̂ꂩ
	// tXN[łOK
	while( hWnd != NULL ){
		if( ! ::IsWindowVisible(hWnd) )
			break;

		if( IsWindowFullScreen(hWnd, &r) )
			return true;
		
		hWnd = ::GetWindow(hWnd, GW_OWNER);
	}

	return false;
}

// tXN[ƊJn
void CIdlePlugin::EnterFullScreen()
{
	CCommonService::connection_list clist;
	CCommonService::connection_list::iterator i;

	// RlNVƂConEnterFullScreen
	NSM_SERVICE.GetConnectionList(clist);
	for( i = clist.begin(); i != clist.end(); ++i )
		ConEnterFullScreen(*i, IDLE_SETTINGS.GetFullScrState());
}

// AChJn
VOID CIdlePlugin::IdleBegin(const ConditionInfo* pci)
{
	CCommonService::connection_list clist;
	CCommonService::connection_list::iterator i;

	// RlNVƂConnectionIdleBegin
	NSM_SERVICE.GetConnectionList(clist);
	for( i = clist.begin(); i != clist.end(); ++i )
		ConIdleBegin(pci, *i);
}

// AChI
VOID CIdlePlugin::StateEnd()
{
	CCommonService::connection_list clist;
	CCommonService::connection_list::iterator i;

	// RlNVƂConnectionIdleEnd
	NSM_SERVICE.GetConnectionList(clist);
	for( i = clist.begin(); i != clist.end(); ++i )
		ConStateEnd(*i);

	// RlNV}bvS폜
	m_cons.clear();
}

// RlNVtXN[ƊJn
void CIdlePlugin::ConEnterFullScreen(HNsmConnection hCon, int nState)
{
	CConState &s = m_cons[hCon];

	if( ! s.IsIdling() ){
		SaveOrig(hCon, s);
	}

	// OZbgEԕύX
	if( ! s.IsFix() ){
		SetUserName(hCon, s.GetOrigName());
		SetUserStatus(hCon, nState);

		if( IDLE_SETTINGS.IsEnableRecoverName() )
			DeleteNameChangeRecord(hCon);
	}
}

// RlNVAChJn
VOID CIdlePlugin::ConIdleBegin(const ConditionInfo* pci,
				HNsmConnection hCon)
{
	CConState &s = m_cons[hCon];

	if( ! s.IsIdling() ){
		SaveOrig(hCon, s);
	}

	// ԕύXEOύX
	if( ! s.IsFix() ){
		std::wstring newName;
		bool bEnableChange;

		bEnableChange = GetIdleUserName(&newName, s.GetOrigName(), pci->str.c_str(), pci->nPos);
		if( bEnableChange )
			SetUserName(hCon, newName.c_str());
		SetUserStatus(hCon, pci->nState);

		if( IDLE_SETTINGS.IsEnableRecoverName() )
			SaveNameChange(hCon, s.GetOrigName(), newName.c_str());
	}
}

// RlNVAChI
VOID CIdlePlugin::ConStateEnd(HNsmConnection hCon)
{
	ConStateMap::iterator i = m_cons.find(hCon);

	if( i == m_cons.end() ) return;
	if( i->second.IsFix() ) return;

	SetUserStatus(hCon, i->second.GetOrigState());
	SetUserName(hCon, i->second.GetOrigName());

	if( IDLE_SETTINGS.IsEnableRecoverName() )
		DeleteNameChangeRecord(hCon);
}

//*******************************************************************
// static o
//*******************************************************************

void CIdlePlugin::SaveNameChange(HNsmConnection hCon, LPCWSTR origName, LPCWSTR newName)
{
	char proto[256], acc[256];

	NSM_SERVICE.GetProtocol(hCon, proto, sizeof(proto));
	NSM_SERVICE.GetUserAccount(hCon, acc, sizeof(acc));

	IDLE_SETTINGS.SaveNameChange(proto, acc, origName, newName);
}

void CIdlePlugin::DeleteNameChangeRecord(HNsmConnection hCon)
{
	char proto[256], acc[256];

	NSM_SERVICE.GetProtocol(hCon, proto, sizeof(proto));
	NSM_SERVICE.GetUserAccount(hCon, acc, sizeof(acc));

	IDLE_SETTINGS.DeleteNameChangeRecord(proto, acc);
}

void CIdlePlugin::RestoreNameChange(HNsmConnection hCon)
{
	char proto[256], acc[256];
	wchar_t currentName[1024];
	std::wstring prevName, chkName;
	bool bExists;

	NSM_SERVICE.GetProtocol(hCon, proto, sizeof(proto));
	NSM_SERVICE.GetUserAccount(hCon, acc, sizeof(acc));

	bExists = IDLE_SETTINGS.GetNameChangeRecord(proto, acc, &prevName, &chkName);

	if( bExists ){
		NSM_SERVICE.GetUserName(hCon, currentName,
			sizeof(currentName)/sizeof(currentName[0]));

		if( currentName == chkName )
			NSM_SERVICE.SetUserName(hCon, prevName.c_str());

		IDLE_SETTINGS.DeleteNameChangeRecord(proto, acc);
	}
}

// IWiԕۑ
void CIdlePlugin::SaveOrig(HNsmConnection hCon, CConState &s)
{
	int state = NSM_SERVICE.GetUserStatus(hCon);
	char proto[256];

	NSM_SERVICE.GetProtocol(hCon, proto, sizeof(proto));

	if( (! IDLE_SETTINGS.IsEnableProto(proto))
		|| (state == NMST_OFFLINE)
		|| (state == NMST_HIDDEN)
		|| (IDLE_SETTINGS.IsOnlineOnly() && (state != NMST_ONLINE))
	){
		s.SetFix();
	}else{
		wchar_t buf[1024];
		int len = NSM_SERVICE.GetUserName(hCon, buf, sizeof(buf));
		buf[len] = '\0';
		s.SetOrigState(state);
		s.SetOrigName(buf);
	}
}

// [U擾
bool CIdlePlugin::GetIdleUserName(std::wstring* out, LPCWSTR name, LPCWSTR str, int pos)
{
	if( pos == AT_PREFIX ){
		*out = str;
		*out += name;
	}else if( pos == AT_POSTFIX ){
		*out = name;
		*out += str;
	}else if( pos == AT_ALL ){
		*out = str;
	}else{
		return false;
	}

	return true;
}

// [UύX
void CIdlePlugin::SetUserName(HNsmConnection hCon, LPCWSTR name)
{
	char proto[256];
	wchar_t curName[1024];

	NSM_SERVICE.GetUserName(hCon, curName, 1024);
	if( wcscmp(curName, name) != 0 ){
		NSM_SERVICE.GetProtocol(hCon, proto, sizeof(proto));
		NSM_SERVICE.ChangeUserName(proto, hCon, name);
	}
}

// [UԕύX
void CIdlePlugin::SetUserStatus(HNsmConnection hCon, int nStatus)
{
	char proto[256];

	if( nStatus != NSM_SERVICE.GetUserStatus(hCon) ){
		NSM_SERVICE.GetProtocol(hCon, proto, sizeof(proto));
		NSM_SERVICE.ChangeUserStatus(proto, hCon, (MemberStatus)nStatus);
	}
}

bool CIdlePlugin::IsWindowFullScreen(HWND hWnd, const RECT* pDesktopRect)
{
	if( hWnd == NULL )
		return false;

DEBUG_CODE(
{
	RECT r;
	::GetWindowRect(hWnd, &r);
	_RPT1(_CRT_WARN, "HWND=%p ", hWnd);
	_RPT1(_CRT_WARN, " isZoom=%d", ::IsZoomed(hWnd));
	_RPT1(_CRT_WARN, " style=%p", ::GetWindowLong(hWnd, GWL_STYLE));
	_RPT4(_CRT_WARN, " wnd=(%d,%d)-(%d,%d)", r.left, r.top, r.right, r.bottom);
	_RPT0(_CRT_WARN, "\n");
}
)

	// (! WS_THICKFRAME) || (WS_POPUP && ! WS_DLGFRAME) łȂ΂ȂȂ
	{
		DWORD style = ::GetWindowLong(hWnd, GWL_STYLE);
		if( ((style & WS_THICKFRAME) != 0)
			&& ((style & (WS_POPUP | WS_DLGFRAME)) != WS_POPUP) )
			return false;
	}

	// EChE`̎擾
	RECT fgRect;
	{
		BOOL fgRet;
		fgRet = ::GetWindowRect(hWnd, &fgRect);
		if( ! fgRet )
			return false;
	}

	// EChE`fXNgbv`ȏłOK
	return (fgRect.top <= pDesktopRect->top)
		&& (fgRect.bottom >= pDesktopRect->bottom)
		&& (fgRect.left <= pDesktopRect->left)
		&& (fgRect.right >= pDesktopRect->right);
}


//*******************************************************************
// R[obN֐
//*******************************************************************

VOID CALLBACK CIdlePlugin::TimerProc(HWND /*hWnd*/, UINT /*uMsg*/, UINT_PTR /*idEvent*/, DWORD /*dwTime*/)
{
	CIdlePlugin::GetInstance().OnTimer();
}
