#include "StdAfx.h"
#include <algorithm>
#include <sstream>
#include ".\idlesettings.h"
#include "OptionDlg.h"
#include "general_strings.h"
#include "util.h"
#include "mleak.h"

const static char INI_SETTING[]			= "Setting";
const static char INI_CHANGERECORD[]	= "ChangeRecord";

const static char INI_ENABLE[]			= "Enable";
const static char INI_FULLSCRSTATE[]	= "FullScrState";
const static char INI_CONDITIONS[]		= "Conditions";
const static char INI_ONLINEONLY[]		= "OnlineOnly";
const static char INI_ENABLES[]			= "EnableProtocols";
const static char INI_RECOVERNAME[]		= "RecoverName";

const static char COND_DELIMITER[]		= " .";
const static char DEFAULT_PROTOCOLS[]	= "MSNP8/YMSG/ICQ/";

CIdleSettings CIdleSettings::m_singleton(PLUGIN_NAME);

//*******************************************************************
// RXgN^EfXgN^
//*******************************************************************
CIdleSettings::CIdleSettings(LPCTSTR pluginName)
: COptionSettings(pluginName)
{
	::InitializeCriticalSection(&m_lock);
	ReadSettings();
}

CIdleSettings::~CIdleSettings(void)
{
	::DeleteCriticalSection(&m_lock);

	CondList::iterator i;

	for( i = m_conds.begin(); i != m_conds.end(); ++i )
		delete *i;
}

//*******************************************************************
// ݒt@Cǂݏ
//*******************************************************************
void CIdleSettings::ReadSettings()
{
	char key[16];
	ConditionInfo* pci;
	tstring cond;
	std::string str;
	std::string enables;
	LPCSTR st, ed;

	m_bEnable = (m_ini.ReadInt(INI_SETTING, INI_ENABLE, 1) != 0);
	m_nFullScrState = m_ini.ReadInt(INI_SETTING, INI_FULLSCRSTATE, NMST_BUSY + 1);
	m_bOnlineOnly = (m_ini.ReadInt(INI_SETTING, INI_ONLINEONLY, 1) != 0);
	m_bRecoverName = (m_ini.ReadInt(INI_SETTING, INI_RECOVERNAME, 1) != 0);
	m_ini.ReadString(enables, INI_SETTING, INI_ENABLES, DEFAULT_PROTOCOLS);

	if( (unsigned int)m_nFullScrState >= STATUS_STRINGS_NUM )
		m_nFullScrState = NMST_BUSY + 1;

	st = enables.c_str();
	for(;;){
		ed = strchr(st, '/');
		if( ed == NULL ) break;
		str.assign(st, ed-st);
		m_enables.push_back(str);
		st = ed + 1;
	}

	pci = new ConditionInfo;
	for(int i = 0; ; i++){
		_itoa_s(i, key, 10);
		m_ini.ReadString(cond, INI_CONDITIONS, key, NULL_STRING);

		st = cond.c_str();
		ed = strchr(st, ' ');
		if( ed == NULL ) break;
		pci->nIdle = atoi(st);

		st = ed + 1;
		ed = strchr(st, ' ');
		if( ed == NULL ) break;
		pci->nState = atoi(st);

		st = ed + 1;
		ed = strchr(st, ' ');
		if( ed == NULL ) break;
		pci->nPos = atoi(st);

		st = ed + 1;
		ed = strstr(st, COND_DELIMITER);
		if( ed == NULL ) break;
		str.assign(st, ed-st);
		HexDec(pci->str, str.c_str());

		m_conds.push_back(pci);
		pci = new ConditionInfo;
	}
	delete pci;

	SortCondList(m_conds);
}

void CIdleSettings::WriteSettings()
{
	int i, last;
	char key[16];
	std::string str;
	std::string enables;

	m_ini.DeleteSection(INI_CONDITIONS);

	for( i = 0, last = m_enables.size(); i < last; ++i ){
		enables += m_enables[i];
		enables += "/";
	}
	m_ini.Write(INI_SETTING, INI_ENABLE, m_bEnable);
	m_ini.Write(INI_SETTING, INI_FULLSCRSTATE, m_nFullScrState);
	m_ini.Write(INI_SETTING, INI_ONLINEONLY, m_bOnlineOnly);
//  m_bRecoverName ͍ċN܂ŕێKv
//	m_ini.Write(INI_SETTING, INI_RECOVERNAME, m_bRecoverName);
	m_ini.Write(INI_SETTING, INI_ENABLES, enables.c_str());

	for( i = 0, last = m_conds.size(); i < last; ++i ){
		const ConditionInfo &ci = *m_conds[i];
		std::ostringstream oss;
		HexEnc(str, ci.str.c_str());
		_itoa_s(i, key, 10);
		oss << ci.nIdle << SPACE_DELIMITER;
		oss << ci.nState << SPACE_DELIMITER;
		oss << ci.nPos << SPACE_DELIMITER;
		oss << str << COND_DELIMITER;
		m_ini.Write(INI_CONDITIONS, key, oss.str().c_str());
	}
}

//*******************************************************************
// ̑
//*******************************************************************
int CIdleSettings::ShowOptionDlg(HINSTANCE hInst, HWND hParentWnd)
{
	COptionDlg dlg;
	CondList list;
	CondList::iterator i;
	bool bRecoverName;

	for( i = m_conds.begin(); i != m_conds.end(); ++i )
		list.push_back(new ConditionInfo(**i));

	// m_bRecoverName ͍ċN܂ŕێ
	bRecoverName = (m_ini.ReadInt(INI_SETTING, INI_RECOVERNAME, 1) != 0);

	dlg.SetEnablePlugin(m_bEnable);
	dlg.SetFullScrState(m_nFullScrState);
	dlg.SetOnlineOnly(m_bOnlineOnly);
	dlg.SetEnableRecoverName(bRecoverName);
	dlg.SetEnableProtocols(m_enables);
	dlg.SetCond(list);

	if( dlg.DoModal(hInst, hParentWnd) == IDOK ){
		SortCondList(list);

		Lock();
		for( i = m_conds.begin(); i != m_conds.end(); ++i )
			delete *i;

		m_bEnable = dlg.GetEnablePlugin();
		m_nFullScrState = dlg.GetFullScrState();
		m_bOnlineOnly = dlg.GetOnlineOnly();
		dlg.GetEnableProtocols(m_enables);
		m_conds = list;
		Unlock();

		WriteSettings();

		// m_bRecoverName ͍ċN܂ŕێ
		m_ini.Write(INI_SETTING, INI_RECOVERNAME, dlg.GetEnableRecoverName());

		return 1;
	}

	for( i = list.begin(); i != list.end(); ++i )
		delete *i;

	return 1;
}

void CIdleSettings::SortCondList(CondList &list) const
{
	std::sort(list.begin(), list.end(), CondListSortFunc);
}

const ConditionInfo* CIdleSettings::GetNowCondition(UINT idletime) const
{
	CondList::const_iterator i;
	for( i = m_conds.begin(); i != m_conds.end(); ++i ){
		if( (*i)->nIdle >= (int)idletime )
			return *i;
	}
	return NULL;
}

const ConditionInfo* CIdleSettings::GetNextCondition(UINT idletime) const
{
	CondList::const_iterator i;
	for( i = m_conds.begin(); i != m_conds.end(); ++i ){
		if( (unsigned int)(*i)->nIdle == idletime )
			return *i;
	}
	return NULL;
}

bool CIdleSettings::CondListSortFunc(const CondList::value_type &a,
		const CondList::value_type &b)
{
	return b->nIdle > a->nIdle;
}

void CIdleSettings::HexEnc(std::string &out, LPCWSTR in) const
{
	const static char b2h[] = "0123456789ABCDEF";

	out.erase();
	for( LPCWSTR pos = in; *pos; ++pos ){
		out += b2h[(*pos >> 0) & 0xF];
		out += b2h[(*pos >> 4) & 0xF];
		out += b2h[(*pos >> 8) & 0xF];
		out += b2h[(*pos >> 12) & 0xF];
	}
}

void CIdleSettings::HexDec(std::wstring &out, LPCSTR in) const
{
	wchar_t c = 0;
	int count = 0;

	out.erase();
	for( LPCSTR pos = in; *pos; ++pos ){
		c |= h2b(*pos) << (count * 4);;
		if( count == 3 ){
            out += c;
			c = 0;
			count = 0;
		}else{
			count++;
		}
	}
}

bool CIdleSettings::IsEnableProto(LPCSTR proto) const
{
	StringList::const_iterator i;
	for( i = m_enables.begin(); i != m_enables.end(); ++i ){
		if( *i == proto ) return true;
	}
	return false;
}


void CIdleSettings::SaveNameChange(LPCSTR proto, LPCSTR acc,
						LPCWSTR origName, LPCWSTR newName)
{
	std::string key;
	std::string value;

	key = "/";
	key += proto;
	key += "/";
	key += acc;

	UnicodeToHex(origName, &value);
	m_ini.Write(INI_CHANGERECORD, ("O" + key).c_str(), value.c_str());

	UnicodeToHex(newName, &value);
	m_ini.Write(INI_CHANGERECORD, ("N" + key).c_str(), value.c_str());
}

void CIdleSettings::DeleteNameChangeRecord(LPCSTR proto, LPCSTR acc)
{
	std::string key;
	std::string value;

	key = "/";
	key += proto;
	key += "/";
	key += acc;

	m_ini.Delete(INI_CHANGERECORD, ("O" + key).c_str());
	m_ini.Delete(INI_CHANGERECORD, ("N" + key).c_str());
}

bool CIdleSettings::GetNameChangeRecord(LPCSTR proto, LPCSTR acc,
						std::wstring* origName, std::wstring* newName)
{
	std::string key;
	std::string value;

	key = "/";
	key += proto;
	key += "/";
	key += acc;

	m_ini.ReadString(value, INI_CHANGERECORD, ("O" + key).c_str(), "0");
	if( value == "0" ) return false;
	HexToUnicode(value.c_str(), origName);

	m_ini.ReadString(value, INI_CHANGERECORD, ("N" + key).c_str(), "0");
	if( value == "0" ) return false;
	HexToUnicode(value.c_str(), newName);

	return true;
}

