#pragma once
#include "optionsettings.h"
#include "etc.h"

//********************************************
#define IDLE_SETTINGS (CIdleSettings::GetInstance())
//********************************************

class CIdleSettings :
	public COptionSettings
{
// singleton -->
private:
	static CIdleSettings m_singleton;

private:
	CIdleSettings(LPCTSTR pluginName);
	~CIdleSettings(void);

public:
	static CIdleSettings& GetInstance()
	{
		return m_singleton;
	}
// <-- singleton


private:
	CRITICAL_SECTION m_lock;
	bool m_bEnable;
	bool m_bOnlineOnly;
	bool m_bRecoverName;
	int m_nFullScrState;
	CondList m_conds;
	StringList m_enables;

public:
	int ShowOptionDlg(HINSTANCE hInst, HWND hParentWnd);
	const ConditionInfo* GetNowCondition(UINT idletime) const;
	const ConditionInfo* GetNextCondition(UINT idletime) const;
	bool IsEnableProto(LPCSTR proto) const;

	void SaveNameChange(LPCSTR proto, LPCSTR acc, LPCWSTR origName, LPCWSTR newName);
	void DeleteNameChangeRecord(LPCSTR proto, LPCSTR acc);
	bool GetNameChangeRecord(LPCSTR proto, LPCSTR acc,
		std::wstring* origName, std::wstring* newName);

	void LockCond()
	{ Lock(); }
	void UnlockCond()
	{ Unlock(); }
	bool IsEnable() const
	{ return m_bEnable; }
	bool IsOnlineOnly() const
	{ return m_bOnlineOnly; }
	bool IsEnableFullScr() const
	{ return m_nFullScrState != 0; }
	int GetFullScrState() const
	{ return m_nFullScrState - 1; }
	bool IsEnableRecoverName(void) const
	{ return m_bRecoverName; }

private:
	void ReadSettings(void);
	void WriteSettings(void);
	void SortCondList(CondList &list) const;
	void HexEnc(std::string &out, LPCWSTR in) const;
	void HexDec(std::wstring &out, LPCSTR in) const;

	void Lock()
	{ ::EnterCriticalSection(&m_lock); }
	void Unlock()
	{ ::LeaveCriticalSection(&m_lock); }

	int h2b(unsigned char c) const
	{
		if( c < '0' ) return 0;
		else if( c <= '9' ) return c - '0';
		else if( c < 'A' ) return 0;
		else if( c <= 'F' ) return c - 'A' + 10;
		else return 0;
	}

private:
	static bool CondListSortFunc(const CondList::value_type &a, const CondList::value_type &b);
};
